<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui version="31" name="Konqueror" >
 <ToolBar newline="true" name="mainToolBar">
  <text>Main Toolbar</text>
  <Merge/>
  <Action name="splitviewh" />
 </ToolBar>
</gui>
