#ifndef _KDEUI_TESTS_KMAINWINDOWRESTORETEST_H_
#define _KDEUI_TESTS_KMAINWINDOWRESTORETEST_H_

#include <kmainwindow.h>

class MainWin1 : public KMainWindow
{
    Q_OBJECT
public:
    MainWin1()
        : KMainWindow()
    {
    }
    ~MainWin1() override
    {
    }
};

class MainWin2 : public KMainWindow
{
    Q_OBJECT
public:
    MainWin2()
        : KMainWindow()
    {
    }
    ~MainWin2() override
    {
    }
};

class MainWin3 : public KMainWindow
{
    Q_OBJECT
public:
    MainWin3()
        : KMainWindow()
    {
    }
    ~MainWin3() override
    {
    }
};

class MainWin4 : public KMainWindow
{
    Q_OBJECT
public:
    MainWin4()
        : KMainWindow()
    {
    }
    ~MainWin4() override
    {
    }
};

class MainWin5 : public KMainWindow
{
    Q_OBJECT
public:
    MainWin5()
        : KMainWindow()
    {
    }
    ~MainWin5() override
    {
    }
};

class MainWin6 : public KMainWindow
{
    Q_OBJECT
public:
    MainWin6()
        : KMainWindow()
    {
    }
    ~MainWin6() override
    {
    }
};

#endif // _KDEUI_TESTS_KMAINWINDOWRESTORETEST_H_
