<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="krichtexteditor" version="14">

<MenuBar>
  <Menu noMerge="1" name="format"><text>F&amp;ormat</text>
    <Action name="format_text_bold"/>
    <Action name="format_text_italic"/>
    <Action name="format_text_underline"/>
    <Action name="format_text_strikeout"/>
    <Action name="format_text_foreground_color"/>
    <Action name="format_text_background_color"/>
    <Action name="format_text_superscript"/>
    <Action name="format_text_subscript"/>
    <Separator/>
    <Action name="format_heading_level"/>
    <Separator/>
    <Menu name="alignment"><text>&amp;Alignment</text>
       <Action name="format_align_left"/>
       <Action name="format_align_center"/>
       <Action name="format_align_right"/>
       <Action name="format_align_justify"/>
    </Menu>
  <Separator lineSeparator="true"/>
  <Action name="format_list_style"/>
  <Action name="format_list_indent_more"/>
  <Action name="format_list_indent_less"/>
  <Separator lineSeparator="true"/>
  <Action name="insert_horizontal_rule"/>
  <Action name="manage_link"/>
  <Action name="format_painter"/>
  </Menu>
</MenuBar>

<ToolBar name="textToolBar" iconText="icononly"><text>Text Toolbar</text>
  <Action name="format_text_bold"/>
  <Action name="format_text_italic"/>
  <Action name="format_text_underline"/>
  <Action name="format_text_strikeout"/>
  <Action name="format_text_foreground_color"/>
  <Action name="format_text_background_color"/>
  <Separator lineSeparator="true"/>
  <Action name="format_font_family"/>
  <Action name="format_font_size"/>
  <Separator lineSeparator="true"/>
  <Action name="format_text_superscript"/>
  <Action name="format_text_subscript"/>
</ToolBar>
<ToolBar name="formatToolBar" iconText="icononly"><text>Format Toolbar</text>
  <Action name="format_heading_level"/>
  <Separator lineSeparator="true"/>
  <Action name="format_align_left"/>
  <Action name="format_align_center"/>
  <Action name="format_align_right"/>
  <Action name="format_align_justify"/>
  <Separator lineSeparator="true"/>
  <Action name="format_list_style"/>
  <Action name="format_list_indent_more"/>
  <Action name="format_list_indent_less"/>
  <Separator lineSeparator="true"/>
  <Action name="insert_horizontal_rule"/>
  <Action name="manage_link"/>
  <Action name="format_painter"/>
  <Action name="action_to_plain_text"/>
</ToolBar>

</gui>
