<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="windowtest" version="1">
	
	<MenuBar>
		<Menu name="file" noMerge="1">
			<text>&amp;File</text>
			<Action name="filenew" />
			<Action name="fileopen" />
			<Action name="filefloppy" />
			<Action name="fileprint" />
			<Separator lineSeparator="true"/>
			<Action name="exit"/>
		</Menu>
		<Menu name="file2">
			<text>F&amp;ile2</text>
			<Action name="filenew2" />
			<Action name="fileopen2" />
			<Separator />
			<Action name="filefloppy2" />
			<Action name="fileprint2" />
			<Separator />
			<Action name="radioButton1" />
			<Action name="radioButton2" />
			<Action name="radioButton3" />
			<Action name="radioButton4" />
		</Menu>
		<Menu name="settings">
			<text>&amp;Settings</text>
			<Merge name="StandardToolBarMenuHandler" append="show_merge"/>
		</Menu>
	</MenuBar>

	
	
	<ToolBar name="mainToolBar"><text>Toolbar 1</text>
		<Action name="filenew" />
		<Action name="fileopen" />
		<Action name="filefloppy" />
		<Action name="fileprint" />
		<Action name="combobox" />
		<Action name="location" />
		<Action name="exit" />
	</ToolBar>
	
	
	<ToolBar name="AnotherToolbar"><text>Toolbar 2</text>
		<Action name="filenew2" />
		<Action name="fileopen2" />
		<Separator />
		<Action name="filefloppy2" />
		<Action name="fileprint2" />
		<Separator />
		<Action name="radioButton1" />
		<Action name="radioButton2" />
		<Action name="radioButton3" />
		<Action name="radioButton4" />
	</ToolBar>

</gui>
