// SPDX-FileCopyrightText: 2024 Manuel Alcaraz Zambrano <manuelalcarazzam@gmail.com>
// SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL

#pragma once

#include <QMetaMethod>

#include <KAboutApplicationDialog>
#include <KAboutPluginDialog>
#include <KActionCategory>
#include <KActionCollection>
#include <KBugReport>
#include <KEditToolBar>
#include <KHelpMenu>
#include <KKeySequenceWidget>
#include <KMainWindow>
#include <KShortcutWidget>
#include <KShortcutsDialog>
#include <KShortcutsEditor>
#include <KToggleToolBarAction>
#include <KToolBar>
#include <KToolTipHelper>
#include <KUndoActions>
#include <KXMLGUIBuilder>
#include <KXMLGUIClient>
#include <KXMLGUIFactory>
#include <KXmlGuiWindow>
